# plot prediction to highlight the artefact
plot_artefact <- function(dataset,pred,name,training=NULL,subsample=NULL){
  filename <- file.path("~",paste0(name,".tiff"))
  grDevices::tiff(filename = filename, width=100,height=100,
        units = "mm", pointsize = 12,
        compression = "lzw",
        bg = "transparent", res = 300)
  if (!is.null(training)) {
    plot( pred, dataset$y, xlim = c(min(dataset$y,pred), max(dataset$y,pred)), ylim = c(min(dataset$y,pred), max(dataset$y,pred)), pch = 19, cex = 0.6, xlab = "prediction", ylab = "output",
    panel.first = rect(training[1],-1e6, training[2], 1e6, col='grey', border=NA))
  } else if(!is.null(subsample)) {
    plot( pred, dataset$y, xlim = c(min(dataset$y,pred), max(dataset$y,pred)), ylim = c(min(dataset$y,pred), max(dataset$y,pred)), pch = 19, cex = 0.6, xlab = "prediction", ylab = "output")
    for (i in 1:length(subsample)) {
      abline(v=subsample[i],col="grey",lwd=0.05)
    }
    par(new=TRUE)
    plot( pred, dataset$y, xlim = c(min(dataset$y,pred), max(dataset$y,pred)), ylim = c(min(dataset$y,pred), max(dataset$y,pred)), pch = 19, cex = 0.6, xlab = "prediction", ylab = "output")
  } else {
    plot( pred, dataset$y, xlim = c(min(dataset$y,pred), max(dataset$y,pred)), ylim = c(min(dataset$y,pred), max(dataset$y,pred)), pch = 19, cex = 0.6, xlab = "prediction", ylab = "output")
  }
  abline(a=0,b=1,lty=2)
  dev.off()

  return(filename)
}

# plot error of prediction to highlight the artefact
plot_error <- function(dataset,pred,name,training=NULL,subsample=NULL){
  filename <- file.path("~",paste0(name,".tiff"))
  grDevices::tiff(filename = filename, width=100,height=100,
        units = "mm", pointsize = 12,
        compression = "lzw",
        bg = "transparent", res = 300)

  if (!is.null(training)) {
    plot(pred, (dataset$y-pred), xlim = c(min(dataset$y,pred), max(dataset$y,pred)), ylim = c(min((dataset$y-pred)), max((dataset$y-pred))), col="purple",pch = 19, cex = 0.6, xlab = "prediction", ylab = "error",
    panel.first = rect(training[1],-1e6, training[2], 1e6, col='grey', border=NA))
  } else if(!is.null(subsample)) {
    plot(pred, (dataset$y-pred), xlim = c(min(dataset$y,pred), max(dataset$y,pred)), ylim = c(min((dataset$y-pred)), max((dataset$y-pred))), col="purple",pch = 19, cex = 0.6, xlab = "prediction", ylab = "error")
    for (i in 1:length(subsample)) {
      abline(v=subsample[i],col="grey",lwd=0.05)
    }
    par(new=TRUE)
    plot(pred, (dataset$y-pred), xlim = c(min(dataset$y,pred), max(dataset$y,pred)), ylim = c(min((dataset$y-pred)), max((dataset$y-pred))), col="purple",pch = 19, cex = 0.6, xlab = "prediction", ylab = "error")
  } else {
    plot(pred, (dataset$y-pred), xlim = c(min(dataset$y,pred), max(dataset$y,pred)), ylim = c(min((dataset$y-pred)), max((dataset$y-pred))), col="purple",pch = 19, cex = 0.6, xlab = "prediction", ylab = "error")
  }
  abline(h=0,lty=2)

  dev.off()
  return(filename)
}

#############################
# HIGHLIGHT RFreg artefact  #
#############################
# Adaptation from http://freerangestats.info/blog/2016/12/10/extrapolation
set.seed(123) # for reproducibility
library(ranger)   # for random forests

# full and perfect dataset (truth: no error)
# assumption: distribution uniform of the value
x0 <- seq(1,300,0.1)
y <- 1/4 * x0

dataset <- data.frame(x0=x0, y=y)

# a subset of the dataset used as training dataset
dataset_training <- dataset[1001:2000,]

# fit model on the training dataset
mod_rf <- ranger(y ~ x0, data=dataset_training)

# predict on the whole dataset
fc_rf <- predict(mod_rf, data = dataset)

# plot
#plot_artefact(dataset=dataset,pred=fc_rf$predictions,name="rf_artefact",training = c(dataset$x0[1],dataset$x0[2000],dataset$x0[1001],dataset$x0[2991]))
plot_artefact(dataset=dataset,pred=fc_rf$predictions,name="rf_artefact",training = c(dataset_training$y[1],dataset_training$y[nrow(dataset_training)]))
plot_error(dataset=dataset,pred=fc_rf$predictions,name="rf_artefact_eps",training = c(dataset_training$y[1],dataset_training$y[nrow(dataset_training)]))
