###########################################################################################
# HIGLIGHT RFreg structural error:                                                        #
# -> with non-uniform pdf of the multi-input, complex y=f(x), and sub-sample              #
###########################################################################################
seed <- 444 #111, 222, 333
set.seed(seed) # for reproducibility

# full and perfect dataset (truth: no error)
# assumption: distribution exponential of the value
x0 <- sort(rexp(2991,rate=0.01))
x1 <- sort(rexp(2991,rate=0.015))
x2 <- sort(rexp(2991,rate=0.02))
x3 <- sort(rexp(2991,rate=0.025))
x4 <- sort(rexp(2991,rate=0.03))
x5 <- sort(rexp(2991,rate=0.035))
x6 <- sort(rexp(2991,rate=0.04))
x7 <- sort(rexp(2991,rate=0.045))
x8 <- sort(rexp(2991,rate=0.05))
x9 <- sort(rexp(2991,rate=0.055))

x0 <- x0*300/max(x0)
x1 <- x1*300/max(x1)
x2 <- x2*300/max(x2)
x3 <- x3*300/max(x3)
x4 <- x4*300/max(x4)
x5 <- x5*300/max(x5)
x6 <- x6*300/max(x6)
x7 <- x7*300/max(x7)
x8 <- x8*300/max(x8)
x9 <- x9*300/max(x9)
x9_bis <- x9
x9_bis[floor(x9) %% 5 == 0] <- 1
x9_bis[floor(x9) %% 5 != 0] <- 50
x9 <- x9_bis

y <-   (x0 + x1 + x2 + log(x3) - x4 + x5 / x6 + x7 + log(x8) / x9)
y <- y*75/max(y)

dataset <- data.frame(x0=x0, x1=x1, x2=x2, x3=x3, x4=x4, x5=x5, x6=x6, x7=x7, x8=x8, x9=x9, y=y)

# a subset of the dataset used as training dataset
indice <- sample.int(2991,replace=FALSE) #randomize
dataset_training <- dataset[indice[1:1000],]

# fit model on the training dataset
mod_rf <- ranger::ranger(y ~ x0+x1+x2+x3+x4+x5+x6+x7+x8+x9, data=dataset_training)

# predict on the whole dataset
fc_rf <- stats::predict(mod_rf, data = dataset)


# wager2014
mod_wager2014 <- ranger::ranger(y ~ x0+x1+x2+x3+x4+x5+x6+x7+x8+x9, data=dataset_training,importance='impurity',keep.inbag=TRUE)
fc_wager2014 <- cipred::pred_rfci_wager2014_prime(mod_wager2014,dataset,"y")

filename <- file.path("~",paste0(paste0("rf_artefact_multix_complex_sampling1000_seed_",seed),"_wager2014.tiff"))
grDevices::tiff(filename = filename, width=100,height=100,
      units = "mm", pointsize = 12,
      compression = "lzw",
      bg = "transparent", res = 300)
p <- ggplot2::ggplot(fc_wager2014, ggplot2::aes(prediction)) +
     ggplot2::geom_ribbon(ggplot2::aes(ymin = lower_pred_0_05-prediction, ymax = upper_pred_0_05-prediction), fill = "grey60") +
     ggplot2::geom_vline(data=dataset_training, ggplot2::aes(xintercept=y), colour="grey",size=0.05) +
     ggplot2::geom_point(ggplot2::aes(x=prediction,y=obs-prediction), colour="purple",size=0.3) +
     ggplot2::labs(x = "prediction",y = "error") +
     ggplot2::theme(panel.grid.major = ggplot2::element_blank(), panel.grid.minor = ggplot2::element_blank(),
                    panel.background = ggplot2::element_blank(), axis.line = ggplot2::element_line(colour = "black"))
p
grDevices::dev.off()

# lu2019
mod_lu2019 <- cipred::fit_rfci_lu2019(dataset=dataset_training,target="y",predictors=c("x0","x1","x2","x3","x4","x5","x6","x7","x8","x9"),n.cores=1)
fc_lu2019 <- cipred::pred_rfci_lu2019_prime(mod_lu2019,dataset=dataset,target="y",predictors=c("x0","x1","x2","x3","x4","x5","x6","x7","x8","x9"))

filename <- file.path("~",paste0(paste0("rf_artefact_multix_complex_sampling1000_seed_",seed),"_lu2019.tiff"))
grDevices::tiff(filename = filename, width=100,height=100,
      units = "mm", pointsize = 12,
      compression = "lzw",
      bg = "transparent", res = 300)
p <- ggplot2::ggplot(fc_lu2019, ggplot2::aes(prediction)) +
     ggplot2::geom_ribbon(ggplot2::aes(ymin = lower_pred_0_05-prediction, ymax = upper_pred_0_05-prediction), fill = "grey60") +
     ggplot2::geom_vline(data=dataset_training, ggplot2::aes(xintercept=y), colour="grey",size=0.05) +
     ggplot2::geom_point(ggplot2::aes(x=prediction,y=obs-prediction), colour="purple",size=0.3) +
     ggplot2::labs(x = "prediction",y = "error") +
     ggplot2::theme(panel.grid.major = ggplot2::element_blank(), panel.grid.minor = ggplot2::element_blank(),
                    panel.background = ggplot2::element_blank(), axis.line = ggplot2::element_line(colour = "black"))
p
grDevices::dev.off()


# pqqplot
tmp <- rbind(
  cbind(data.frame(ID=rep(1,nrow(fc_wager2014))),fc_wager2014[,c("pvalue","method")]),
  cbind(data.frame(ID=rep(1,nrow(fc_lu2019))),fc_lu2019[,c("pvalue","method")]))

cipred::pqq_plot(df=tmp,DESKTOP=FALSE,name=paste0("synthetic_seed_",seed), path='~')
